@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdcl.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdcl.c,v 1.2 2008/05/12 03:11:27 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -793,10 +793,8 @@ calmaElementSref()
 	    def = calmaFindCell(sname, NULL);
 	}
     }
-    else
-    {
-	def = calmaFindCell(sname, NULL);
-    }
+
+    if (!def) def = calmaFindCell(sname, NULL);
 
     if (DBIsAncestor(def, cifReadCellDef))
     {
