@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.9 2008/01/26 04:19:22 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.10 2008/02/10 19:06:31 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1116,7 +1116,7 @@ calmaWriteContacts(f)
     FILE *f;
 {
     TileType type;
-    TileTypeBitMask tMask;
+    TileTypeBitMask tMask, *rMask;
     CellDef *def, *cellDef;
     Rect area, cliprect;
     int halfwidth, halfsize;
@@ -1137,7 +1137,10 @@ calmaWriteContacts(f)
     /* Decompose stacking types */
     for (type = DBNumUserLayers; type < DBNumTypes; type++)
 	if (TTMaskHasType(&tMask, type))
-	    TTMaskSetMask(&tMask, DBResidueMask(type));
+	{
+	    rMask = DBResidueMask(type);
+	    TTMaskSetMask(&tMask, rMask);
+	}
 
     for (type = TT_SPACE + 1; type < DBNumUserLayers; type++)
     {
