@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.9 2008/01/09 00:51:55 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.10 2008/02/10 19:06:31 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -211,7 +211,7 @@ cifParseLayers(string, style, paintMask, cifMask,spaceOK)
     				   layer?
 				*/
 {
-    TileTypeBitMask curCifMask,curPaintMask;
+    TileTypeBitMask curCifMask, curPaintMask;
     char curLayer[40], *p, *cp;
     TileType paintType;
     int i;
@@ -284,14 +284,18 @@ okpaint:
 	    else
 	    {
 		TileType rtype;
+		TileTypeBitMask *rMask;
 
 		TTMaskSetMask(paintMask, &curPaintMask);
 
 		/* Add residues from '*' notation */
 		if (allResidues)
 		    for (rtype = TT_TECHDEPBASE; rtype < DBNumUserLayers; rtype++)
-			if (TTMaskHasType(DBResidueMask(rtype), paintType))
+		    {
+			rMask = DBResidueMask(rtype);
+			if (TTMaskHasType(rMask, paintType))
 			    TTMaskSetType(paintMask, rtype);
+		    }
 	    } 
 	}
 	else if (!TTMaskEqual(&curCifMask, &DBZeroTypeBits))
