@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.11 2008/01/17 22:26:54 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdCD.c,v 1.15 2008/05/17 21:37:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1613,6 +1613,7 @@ CmdCopy(w, cmd)
     Rect rootBox, newBox;
     Point rootPoint, editPoint;
     CellDef *rootDef;
+    int argpos;
 
     if (cmd->tx_argc > 4)
     {
@@ -1639,37 +1640,34 @@ CmdCopy(w, cmd)
 	    goto copyToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
-	if (indx < 0)
-	    return;
-	if (cmd->tx_argc == 3)
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, FALSE);
+	argpos = (indx < 0) ? 1 : 2;
+
+	if (cmd->tx_argc >= 3)
 	{
-	    if (DBWSnapToGrid != DBW_SNAP_USER)
+	    switch (indx)
 	    {
-		amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-		amounty = amountx;
-	    }
-	    else
-	    {
-		switch (indx)
-		{
-		    case GEO_EAST: case GEO_WEST:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amounty = amountx;
-			break;
-		    case GEO_NORTH: case GEO_SOUTH:
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			amountx = amounty;
-			break;
-		    default:
-			amountx = cmdParseCoord(w, cmd->tx_argv[2], TRUE, FALSE);
-			amounty = cmdParseCoord(w, cmd->tx_argv[2], TRUE, TRUE);
-			break;
-		}
+		case GEO_EAST: case GEO_WEST:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = 0;
+		    break;
+		case GEO_NORTH: case GEO_SOUTH:
+		    amountx = 0;
+		    amounty = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, FALSE);
+		    break;
+		default:
+		    amountx = cmdParseCoord(w, cmd->tx_argv[argpos], TRUE, TRUE);
+		    amounty = cmdParseCoord(w, cmd->tx_argv[cmd->tx_argc - 1],
+				TRUE, FALSE);
+		    break;
 	    }
 	}
 	else
 	{
+	    if (indx < 0) {
+		TxError("Improperly defined copy. . . direction needed.\n");
+		return;
+	    }
 	    amountx = cmdParseCoord(w, "1l", TRUE, TRUE);
 	    amounty = cmdParseCoord(w, "1l", TRUE, FALSE);
 	}
@@ -1697,6 +1695,7 @@ CmdCopy(w, cmd)
 		ydelta = amounty;
 		break;
 	    case GEO_NORTHEAST:
+	    case -2:
 		xdelta = amountx;
 		ydelta = amounty;
 		break;
