@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.18 2008/02/07 06:22:44 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.20 2008/02/18 03:20:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -247,7 +247,7 @@ CmdTech(w, cmd)
 	case TECH_LAYERS:
 	    if (cmd->tx_argc == 4)
 	    {
-		TileTypeBitMask lockmask;
+		TileTypeBitMask lockmask, *rMask;
 		TileType ctype;
 
 		if (!strcmp(cmd->tx_argv[3], "*"))
@@ -262,11 +262,14 @@ CmdTech(w, cmd)
 			    if (DBIsContact(ctype))
 				DBLockContact(ctype);
 		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-			if (TTMaskIntersect(&lockmask, DBResidueMask(ctype)))
+		    {
+			rMask = DBResidueMask(ctype);
+			if (TTMaskIntersect(&lockmask, rMask))
 			{
 			    TTMaskClearType(&DBActiveLayerBits, ctype);
 			    DBLockContact(ctype);
 			}
+		    }
 		}
 		else if (!strcmp(cmd->tx_argv[2], "unlock"))
 		{
@@ -279,9 +282,9 @@ CmdTech(w, cmd)
 		    for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
 		    {
 			TileTypeBitMask testmask;
-			TTMaskAndMask3(&testmask, &DBActiveLayerBits,
-					DBResidueMask(ctype));
-			if (TTMaskEqual(&testmask, DBResidueMask(ctype)))
+			rMask = DBResidueMask(ctype);
+			TTMaskAndMask3(&testmask, &DBActiveLayerBits, rMask);
+			if (TTMaskEqual(&testmask, rMask))
 			{
 			    TTMaskSetType(&DBActiveLayerBits, ctype);
 			    DBUnlockContact(ctype);
@@ -753,24 +756,31 @@ CmdUpsidedown(w, cmd)
  * cmdWhatPrintCell --
  *
  * This command is used to print the cell id of the types encountered in the
- * what command. For each type it only prints the name once.  The routine
- * must be initialized by calling it with NULL args everytime the tile type
- * changes and we start a new search.  (BSI)
+ * what command. For each type it only prints the name once.
  *
  * Results:
  *	Returns 0 to keep the search going.
  *
  * Side Effects:
- *	Prints cell names to the terminal.
+ *	Prints cell names to the terminal.  Adds to a maintained list of
+ *	unique cell IDs, which needs to be free'd by the calling routine.
  * ----------------------------------------------------------------------------
  */
 
+struct linked_id {
+   char *lid_name;
+   struct linked_id *lid_next;
+};
+
+
 int cmdWhatPrintCell(tile, cxp)
    Tile *tile;
    TreeContext *cxp;
 {
+    struct linked_id **lid = (struct linked_id **)cxp->tc_filter->tf_arg;
+    struct linked_id *curlid = *lid;
     char *CurrCellName;
-    char **whatCell = (char **)cxp->tc_filter->tf_arg;
+    char *whatCell;
 
     // Use the id of the cell or its name if top cell
 
@@ -778,10 +788,19 @@ int cmdWhatPrintCell(tile, cxp)
     if ((CurrCellName == NULL) || (CurrCellName[0] == '\0'))
 	CurrCellName = cxp->tc_scx->scx_use->cu_def->cd_name;
 
-    if (*whatCell != CurrCellName)
+    for (curlid = *lid; curlid != NULL; curlid = curlid->lid_next)
+    {
+	whatCell = curlid->lid_name;
+	if (whatCell == CurrCellName)
+	    break;
+    }
+    if (curlid == NULL)
     {
 	TxPrintf(" %s ", CurrCellName);
-	*whatCell = CurrCellName;
+	curlid = (struct linked_id *)mallocMagic(sizeof(struct linked_id));
+	curlid->lid_name = CurrCellName;
+	curlid->lid_next = *lid;
+	*lid = curlid;
     }
     return 0;
 }
@@ -826,10 +845,10 @@ CmdWhat(w, cmd)
     int i, locargc;
     bool foundAny;
     bool doList = FALSE;
-    TileTypeBitMask layers, maskBits;
+    TileTypeBitMask layers, maskBits, *rMask;
     SearchContext scx;
     CellUse *CheckUse;
-    char *whatCell;
+    struct linked_id *lid;
 
 #ifdef MAGIC_WRAPPER
     Tcl_Obj *lobj, *paintobj, *labelobj, *cellobj;
@@ -879,7 +898,10 @@ CmdWhat(w, cmd)
 	for (i = DBNumUserLayers; i < DBNumTypes; i++)
 	{
 	    if (TTMaskHasType(&layers, i))
-		TTMaskSetMask(&layers, DBResidueMask(i));
+	    {
+		rMask = DBResidueMask(i);
+		TTMaskSetMask(&layers, rMask);
+	    }
 	    TTMaskClearType(&layers, i);
 	}
     }
@@ -917,13 +939,18 @@ CmdWhat(w, cmd)
 		{
 		    if (TTMaskHasType(&layers, i))
 		    {
-			whatCell = NULL;
+			lid = NULL;
 			TxPrintf("    %-8s (", DBTypeLongName(i));
 			TTMaskSetOnlyType(&maskBits, i);
 			if (DBIsContact(i)) DBMaskAddStacking(&maskBits);
 			DBTreeSrTiles(&scx, &maskBits, 0, cmdWhatPrintCell,
-				(ClientData)&whatCell);
+				(ClientData)&lid);
 			TxPrintf(")\n");
+			while (lid != NULL)
+			{
+			    freeMagic(lid);
+			    lid = lid->lid_next;
+			}
 		    }
 		}
 	    }
