@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellcopy.c,v 1.7 2008/01/29 02:41:57 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellcopy.c,v 1.8 2008/02/10 19:06:31 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -150,10 +150,11 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
 
     if (DBIsContact(loctype))
     {
-	TileTypeBitMask tmask;
+	TileTypeBitMask tmask, *rMask;
 
-	TTMaskAndMask3(&tmask, DBResidueMask(loctype), &DBActiveLayerBits);
-	if (!TTMaskEqual(&tmask, DBResidueMask(loctype)))
+	rMask = DBResidueMask(loctype);
+	TTMaskAndMask3(&tmask, rMask, &DBActiveLayerBits);
+	if (!TTMaskEqual(&tmask, rMask))
 	{
 	    if (!TTMaskIsZero(&tmask))
 		for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
@@ -569,10 +570,11 @@ dbCopyAllPaint(tile, cxp)
     /* Resolve what type we're going to paint, based on the type and mask */
     if (!TTMaskHasType(typeMask, type))
     {
-	TileTypeBitMask rMask;
+	TileTypeBitMask rMask, *tmask;
 
 	/* Simple case---typeMask has a residue of type on pNum */
-	TTMaskAndMask3(&rMask, typeMask, DBResidueMask(type));
+	tmask = DBResidueMask(type);
+	TTMaskAndMask3(&rMask, typeMask, tmask);
 	TTMaskAndMask(&rMask, &DBPlaneTypes[pNum]);
 	if (!TTMaskIsZero(&rMask))
 	{
