@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.13 2008/02/03 21:23:56 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.15 2008/02/28 21:39:05 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1074,6 +1074,41 @@ dbReadOpen(cellDef, name, setFileName)
     cellDef->cd_flags |= CDAVAILABLE;
     return (f);
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBTestOpen --
+ *
+ * Check whether or not a database file can be found on disk.
+ *
+ * Results:
+ *	Returns TRUE if available, FALSE if not.
+ *
+ * Side effects:
+ *	Full pathname is returned in fullPath (if non-NULL)
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBTestOpen(name, fullPath)
+    char *name;
+    char **fullPath;
+{
+    FILE *f;
+
+    f = PaLockOpen(name, "r", DBSuffix, Path, CellLibPath,
+		fullPath, (bool *)NULL);
+
+    if (f != NULL)
+    {
+	fclose(f);
+	return TRUE;
+    }
+    return FALSE;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -2002,7 +2037,7 @@ DBCellWriteFile(cellDef, f)
     struct writeArg arg;
     int pNum;
     TileType type, stype;
-    TileTypeBitMask typeMask;
+    TileTypeBitMask typeMask, *sMask;
     int reducer;
     char *estring;
 
@@ -2068,8 +2103,11 @@ DBCellWriteFile(cellDef, f)
 	/* have this type as a residue.				*/
 
 	for (stype = DBNumUserLayers; stype < DBNumTypes; stype++)
-	    if (TTMaskHasType(DBResidueMask(stype), type))
+	{
+	    sMask = DBResidueMask(stype);
+	    if (TTMaskHasType(sMask, type))
 		TTMaskSetType(&typeMask, stype);
+	}
 
 	if (DBSrPaintArea((Tile *) NULL, cellDef->cd_planes[pNum],
 		&TiPlaneRect, &typeMask, dbWritePaintFunc, (ClientData) &arg))
@@ -2548,20 +2586,24 @@ dbWritePaintFunc(tile, cdarg)
     TileType type = TiGetType(tile);
 
 #ifdef NONMANHATTAN
+    TileTypeBitMask *rMask, *lMask;
     int dir;
 
     if (IsSplit(tile))      
     { 
+	lMask = DBResidueMask(SplitLeftType(tile));
+	rMask = DBResidueMask(SplitRightType(tile));
+
 	if ((SplitLeftType(tile) == arg->wa_type) ||
 		((SplitLeftType(tile) >= DBNumUserLayers) &&
-		TTMaskHasType(DBResidueMask(SplitLeftType(tile)), arg->wa_type)))
+		TTMaskHasType(lMask, arg->wa_type)))
 	{
 	    type = arg->wa_type;
 	    dir = 0x0;
 	}
 	else if ((SplitRightType(tile) == arg->wa_type) ||
 		((SplitRightType(tile) >= DBNumUserLayers) &&
-		TTMaskHasType(DBResidueMask(SplitRightType(tile)), arg->wa_type)))
+		TTMaskHasType(rMask, arg->wa_type)))
 	{
 	    type = arg->wa_type;
 	    dir = 0x2;
@@ -2573,8 +2615,9 @@ dbWritePaintFunc(tile, cdarg)
 #endif   
     if (type != arg->wa_type)
     {
+	rMask = DBResidueMask(type);
 	if ((type < DBNumUserLayers) ||
-	    	(!TTMaskHasType(DBResidueMask(type), arg->wa_type)))
+	    	(!TTMaskHasType(rMask, arg->wa_type)))
 	    return 0;
 
 	type = arg->wa_type;
