@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.8 2008/01/30 06:26:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtcontact.c,v 1.9 2008/02/10 19:06:31 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -479,10 +479,14 @@ DBMaskAddStacking(mask)
     TileTypeBitMask *mask;
 {
     TileType ttype;
+    TileTypeBitMask *rMask;
 
     for (ttype = DBNumUserLayers; ttype < DBNumTypes; ttype++)
-	if (TTMaskIntersect(DBResidueMask(ttype), mask))
+    {
+	rMask = DBResidueMask(ttype);
+	if (TTMaskIntersect(rMask, mask))
 	    TTMaskSetType(mask, ttype);
+    }
 }
 
 
