@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechtype.c,v 1.4 2006/11/08 01:18:52 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtechtype.c,v 1.8 2008/01/05 19:48:29 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -63,6 +63,8 @@ TileTypeBitMask DBAllButSpaceBits;
 TileTypeBitMask DBAllButSpaceAndDRCBits;
 TileTypeBitMask DBSpaceBits;
 TileTypeBitMask DBUserLayerBits;
+TileTypeBitMask DBActiveLayerBits;	/* Layers that are locked */
+
 
 /* Table of default, builtin planes */
 DefaultPlane dbTechDefaultPlanes[] =
@@ -232,6 +234,9 @@ DBTechInitType()
         TTMaskSetOnlyType(&DBLayerTypeMaskTbl[dtp->dt_type], dtp->dt_type);
     }
 
+    /* Zero the active layers (this mask is inverted later) */
+    TTMaskZero(&DBActiveLayerBits);
+
     /* Hash table of layer aliases, free'ing the allocated type masks */
     HashFreeKill(&DBTypeAliasTable);
     HashInit(&DBTypeAliasTable, 8, HT_STRINGKEYS);
@@ -450,6 +455,14 @@ DBTechAddType(sectionName, argc, argv)
 	cp = dbTechNameAdd(argv[1], (ClientData) DBNumTypes, &dbTypeNameLists);
 	if (cp == NULL)
 	    return FALSE;
+
+	/* Set the lock layer bit if the plane name begins with '-' */
+	if (*argv[0] == LOCKLAYERCHAR)
+	{
+	    TTMaskSetType(&DBActiveLayerBits, DBNumTypes);
+	    argv[0]++;
+	}
+
 	pNum = DBTechNoisyNamePlane(argv[0]);
 	if (pNum < 0)
 	    return FALSE;
@@ -561,6 +574,10 @@ DBTechFinalType()
     /* UserLayerBits includes space */
     TTMaskSetType(&DBUserLayerBits, TT_SPACE);
 
+    /* Set locked layer bitmask */
+    TTMaskCom(&DBActiveLayerBits);
+    TTMaskAndMask(&DBActiveLayerBits, &DBAllButSpaceAndDRCBits);
+
     /* Space is visible on all planes but the subcell plane */
     TTMaskZero(&DBPlaneTypes[PL_CELL]);
     for (pNum = PL_PAINTBASE;  pNum < PL_MAXTYPES;  pNum++)
