@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCtech.c,v 1.39 2008/02/09 17:17:30 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCtech.c,v 1.41 2008/03/04 17:46:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -875,10 +875,14 @@ drcAssign(cookie, dist, next, mask, corner, why, cdist, flags, planeto, planefro
     char *why;
     int flags, planeto, planefrom;
 {
+
+#if 0
     /* Diagnostic */
     if (planeto >= DBNumPlanes) {
 	TxError("Bad plane in DRC assign!\n");
     }
+#endif
+
     (cookie)->drcc_dist = dist;
     (cookie)->drcc_next = next;
     (cookie)->drcc_mask = *mask;
@@ -2286,7 +2290,7 @@ drcEdge(argc, argv)
     ptest = DBTechNoisyNameMask(okTypes, &setM);
     pMaskM = CoincidentPlanes(&setM, ptest);
  
-    if (pMaskM == 0)
+    if (pMaskM == 0 || pMaskM == DBTypePlaneMaskTbl[TT_SPACE])
     {
 	/* Technically it should be illegal to specify simply "space"
 	 * in the types list for a DRC rule, as it is ambiguous.
