@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2spice/ext2spice.c,v 1.5 2008/01/03 07:48:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2spice/ext2spice.c,v 1.6 2008/04/23 05:23:59 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -77,7 +77,7 @@ float esScale = -1.0 ; /* negative if hspice the EFScale/100 otherwise */
 static unsigned short esFormat = SPICE3 ;
 
 int esCapNum, esDevNum, esResNum, esDiodeNum;
-int esNodeNum;  /* just in case we' re extracting spice2 */
+int esNodeNum;  /* just in case we're extracting spice2 */
 int esSbckNum; 	/* used in hspice node name shortening   */
 int esNoModelType;  /* index for device type "None" (model-less device) */
 
@@ -282,8 +282,9 @@ Exttospice_Init(interp)
 #define EXTTOSPC_MERGE		5
 #define EXTTOSPC_EXTRESIST	6
 #define EXTTOSPC_RESISTORTEE	7
-#define EXTTOSPC_SUBCIRCUITS	8
-#define EXTTOSPC_HELP		9
+#define EXTTOSPC_SCALE		8
+#define EXTTOSPC_SUBCIRCUITS	9
+#define EXTTOSPC_HELP		10
 
 void
 CmdExtToSpice(w, cmd)
@@ -321,6 +322,7 @@ CmdExtToSpice(w, cmd)
 	"merge [<type>]		merge parallel transistors",
 	"extresist [on|off]	incorporate information from extresist",
 	"resistor tee [on|off]	model resistor capacitance as a T-network",
+	"scale [on|off]		use .option card for scaling",
 	"subcircuits [on|off]	standard cells become subcircuit calls",
 	"help			print help information",
 	NULL
@@ -397,6 +399,20 @@ CmdExtToSpice(w, cmd)
 	    else esDoResistorTee = FALSE;
 	    break;
 
+	case EXTTOSPC_SCALE:
+	    if (cmd->tx_argc == 2)
+	    {
+		Tcl_SetResult(magicinterp, (esScale < 0) ? "on" : "off", NULL);
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto usage;
+	    idx = Lookup(cmd->tx_argv[2], yesno);
+	    if (idx < 0) goto usage;
+	    else if (idx < 3) esScale = -1.0;
+	    else esScale = 0.0;
+	    break;
+
 	case EXTTOSPC_SUBCIRCUITS:
 	    if (cmd->tx_argc == 2)
 	    {
@@ -472,7 +488,12 @@ CmdExtToSpice(w, cmd)
 			"spice2, spice3, and hspice.", NULL);
 		return;
 	    }
-	    else esFormat = idx;
+	    else
+	    {
+		esFormat = idx;
+		/* By default, use .option to declare scale in HSPICE mode */
+		if (esFormat == HSPICE) esScale = -1.0;
+	    }
 	    break;
 
 	case EXTTOSPC_CTHRESH:
@@ -715,9 +736,10 @@ runexttospice:
 	    (esFormat == SPICE2) ? "SPICE2" :  
 	      ( (esFormat == SPICE3) ? "SPICE3" : "HSPICE" ),  
 	    inName, EFTech);
-    if ( esFormat == HSPICE ) 
+    if (esScale < 0) 
     	fprintf(esSpiceF,".option scale=%gu\n\n", EFScale / 100.0);
-    else esScale = EFScale / 100.0;
+    else
+	esScale = EFScale / 100.0;
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -872,9 +894,10 @@ main(argc, argv)
 	    (esFormat == SPICE2) ? "SPICE2" :  
 	      ( (esFormat == SPICE3) ? "SPICE3" : "HSPICE" ),  
 	    inName, EFTech);
-    if ( esFormat == HSPICE ) 
+    if (esScale < 0) 
     	fprintf(esSpiceF,".option scale=%gu\n\n", EFScale / 100.0);
-    else esScale = EFScale / 100.0;
+    else
+	esScale = EFScale / 100.0;
 
     /* Convert the hierarchical description to a flat one */
     flatFlags = EF_FLATNODES;
@@ -1009,7 +1032,10 @@ spcmainArgs(pargc, pargv)
 	     else if ( strcasecmp(ftmp,"SPICE3") == 0 )
 		esFormat = SPICE3 ;
 	     else if ( strcasecmp(ftmp,"HSPICE") == 0 ) 
+	     {
 		esFormat = HSPICE ;
+		esScale = -1.0;
+	     }
 	     else goto usage;
 	     break;
 	     }
@@ -1468,7 +1494,7 @@ esOutputResistor(dev, hierName, trans, term1, term2, has_model, l, w, dscale)
 	GeoTransRect(trans, &dev->dev_rect, &r);
 	scale = GeoScale(trans);
 
-	if (esFormat == HSPICE)
+	if (esScale < 0) 
 	{
 	    fprintf(esSpiceF, " w=%d l=%d", w * scale, (l * scale) / dscale);
 	    if (sdM != 1.0)
@@ -1709,7 +1735,7 @@ spcdevVisit(dev, hierName, trans)
 
 	    if ((w > 0) && (l > 0))
 	    {
-		if (esFormat == HSPICE)
+		if (esScale < 0)
 		{
 		    fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
 		    if (sdM != 1.0)
@@ -1800,7 +1826,7 @@ spcdevVisit(dev, hierName, trans)
 		GeoTransRect(trans, &dev->dev_rect, &r);
 		scale = GeoScale(trans);
 
-		if (esFormat == HSPICE)
+		if (esScale < 0)
 		{
 		    fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
 		    if (sdM != 1.0)
@@ -1849,7 +1875,7 @@ spcdevVisit(dev, hierName, trans)
 
 	    sdM = getCurDevMult();
 
-	    if (esFormat == HSPICE)
+	    if (esScale < 0)
 	    {
 		fprintf(esSpiceF, " w=%d l=%d", w*scale, l*scale);
 		if (sdM != 1.0)
@@ -2013,14 +2039,14 @@ int spcnAP(node, resClass, scale, sterm, m, outf, w)
 	    return;
 	}
 
-	if ( esFormat == HSPICE )
+	if ( esScale < 0 )
 	     sprintf(fmt,"a%s=%%g p%s=%%g ", sterm, sterm);
 	else sprintf(fmt,"a%s=%%gp p%s=%%gu ", sterm, sterm);
 
 	if ( !esDistrJunct || w == -1 ) goto oldFmt;
 
 	dsc = w/((nodeClient*)node->efnode_client)->m_w.widths[resClass];
-	if ( esFormat == HSPICE )
+	if ( esScale < 0 )
 	   fprintf(outf,fmt,
 		node->efnode_pa[resClass].pa_area*scale*scale*dsc,
 		node->efnode_pa[resClass].pa_perim*scale*dsc);
@@ -2036,7 +2062,7 @@ oldFmt:
 	     beenVisited((nodeClient *)node->efnode_client, resClass) ) 
 		scale = 0 ;
 	else   markVisited((nodeClient *)node->efnode_client, resClass);
-	if ( esFormat == HSPICE )
+	if ( esScale < 0 )
 	   fprintf(outf,fmt,
 		node->efnode_pa[resClass].pa_area*scale*scale/m,
 		node->efnode_pa[resClass].pa_perim*scale/m);
@@ -2058,7 +2084,7 @@ int spcnAPHier(dterm, hierName, resClass, scale, sterm, m, outf)
 	nodeClientHier   *nc ;
 	char fmt[30];
 
-	if ( esFormat == HSPICE )
+	if ( esScale < 0 )
 	     sprintf(fmt,"a%s=%%g p%s=%%g ", sterm, sterm);
 	else sprintf(fmt,"a%s=%%gp p%s=%%gu ", sterm, sterm);
 	if ( node->efnode_client == (ClientData) NULL ) 
@@ -2073,7 +2099,7 @@ int spcnAPHier(dterm, hierName, resClass, scale, sterm, m, outf)
 	    beenVisited((nodeClientHier *)node->efnode_client, resClass) ) 
 		scale = 0;
 	else markVisited((nodeClientHier *)node->efnode_client, resClass);
-	if ( esFormat == HSPICE )
+	if ( esScale < HSPICE )
 	   fprintf(outf,fmt,
 		node->efnode_pa[resClass].pa_area*scale*scale/m,
 		node->efnode_pa[resClass].pa_perim*scale/m);
