@@ -714,6 +714,12 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
     int segp, segn, segc, sege;
     bool isComplex = FALSE;
 
+    /* First, check for MOScap-connected transistors.  In such
+     * cases, one or more extSpecialBounds[] is NULL.  Try to
+     * separate the existing extSpecialBounds[] vectors into 
+     * independent (non-connecting) vectors.
+     */
+
     /* For each segment in the primary list, find the closest
      * segment in the other list which lies on the opposite
      * side of the gate area.  Calculate the length, and check
@@ -1100,30 +1106,7 @@ extOutputTrans(def, transList, outFile)
 	arg.fra_connectsTo = ExtCurStyle->exts_transConn;
 
 	extTransRec.tr_gatenode = (NodeRegion *) extGetRegion(reg->treg_tile);
-	t = TiGetType(reg->treg_tile);
-
-#ifdef NONMANHATTAN
-	if (IsSplit(reg->treg_tile)) {
-
-	    /* ExtFindNeighbors() assumes that split tiles have the TT_SIDE	*/
-	    /* bit set or cleared according to which side of the tile is	*/
-	    /* being searched for neighbors.					*/
-
-	    t = TiGetLeftType(reg->treg_tile);
-	    if (TTMaskHasType(&arg.fra_connectsTo[reg->treg_type], t))
-		TiSetBody(reg->treg_tile, TiGetTypeExact(reg->treg_tile) & (~TT_SIDE));
-	    else {
-		t = TiGetRightType(reg->treg_tile);
-		TiSetBody(reg->treg_tile, TiGetTypeExact(reg->treg_tile) | TT_SIDE);
-	    }
-
-	    /* If the tile is split, we can't depend on its ti_client field	*/
-	    /* pointing to the correct region.					*/
-
-	    /* extTransRec.tr_gatenode = ??? */
-	}
-#endif
-
+	t = reg->treg_type;
 	arg.fra_pNum = DBPlane(t);
 
 	    /* Mark with reg and process each perimeter segment */
@@ -1287,6 +1270,7 @@ extOutputTrans(def, transList, outFile)
 		    arg.fra_each = extAnnularTileFunc;
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
+		    extSeparateBounds(n - 1);	/* Handle MOScaps (if necessary) */
 		    extComputeEffectiveLW(&length, &width, n);
 
 		    /* Free the lists */
@@ -1645,7 +1629,7 @@ extTransPerimFunc(bp)
 	    extTransRec.tr_termlen[thisterm] = 0;
 	    extTransRec.tr_termvector[thisterm].p_x = 0;
 	    extTransRec.tr_termvector[thisterm].p_y = 0;
-	    extTransRec.tr_termpos[thisterm].pnum = DBPlane(TiGetType(bp->b_outside));
+	    extTransRec.tr_termpos[thisterm].pnum = DBPlane(toutside);
 	    extTransRec.tr_termpos[thisterm].pt = bp->b_outside->ti_ll;
 
 	    /* Find the total area of this terminal */
@@ -1720,7 +1704,7 @@ extTransPerimFunc(bp)
      */
 
     /* Don't double-count contact perimeters (added by Tim 1/9/07) */
-    if (bp->b_plane == extTransRec.tr_gatenode->nreg_pnum)
+    if (!DBIsContact(toutside) || (bp->b_plane == extTransRec.tr_gatenode->nreg_pnum))
 	extTransRec.tr_perim += len;
 
     return (0);
@@ -2229,6 +2213,13 @@ extTransFirst(tile, arg)
     reg->treg_area = 0;
     reg->treg_tile = tile;
 
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+	reg->treg_type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
+    else
+#endif
+	reg->treg_type = TiGetTypeExact(tile);
+
     /* Prepend it to the region list */
     reg->treg_next = (TransRegion *) arg->fra_region;
     arg->fra_region = (Region *) reg;
@@ -2247,10 +2238,12 @@ extTransEach(tile, pNum, arg)
 
 #ifdef NONMANHATTAN
     if (IsSplit(tile)) area /= 2;	/* Split tiles are 1/2 area! */
-    else
+    else if (IsSplit(reg->treg_tile))
+    {
 	/* Avoid setting the region's tile pointer to a split tile */
-	if (IsSplit(reg->treg_tile))
-	    reg->treg_tile = tile;
+	reg->treg_tile = tile;
+	reg->treg_type = TiGetTypeExact(tile);
+    }
 #endif
 
     /* Don't double-count area on contacts (added by Tim 1/9/07) */
@@ -2468,16 +2461,19 @@ topside:
             if (IsSplit(tp))
 	    {
         	t = SplitBottomType(tp);
-		tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body & ~TT_SIDE);
-		if (!SplitDirection(tp))
-		    tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body | TT_SIDE);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILEBOTTOM(tp, tilePlaneNum);
+		}
 	    }
             else
 #endif
-	    t = TiGetTypeExact(tp);
-	    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
 	    {
-		PUSHTILE(tp, tilePlaneNum);
+		t = TiGetTypeExact(tp);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILE(tp, tilePlaneNum);
+		}
 	    }
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
@@ -2508,14 +2504,19 @@ leftside:
             if (IsSplit(tp))
 	    {
                 t = SplitRightType(tp);
-		tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body | TT_SIDE);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILERIGHT(tp, tilePlaneNum);
+		}
 	    }
             else
 #endif
-	    t = TiGetTypeExact(tp);
-	    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
 	    {
-		PUSHTILE(tp, tilePlaneNum);
+		t = TiGetTypeExact(tp);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILE(tp, tilePlaneNum);
+		}
 	    }
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
@@ -2547,16 +2548,19 @@ bottomside:
             if (IsSplit(tp))
 	    {
                 t = SplitTopType(tp);
-		tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body & ~TT_SIDE);
-		if (SplitDirection(tp))
-		    tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body | TT_SIDE);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILETOP(tp, tilePlaneNum);
+		}
 	    }
             else
 #endif
-	    t = TiGetTypeExact(tp);
-	    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
 	    {
-		PUSHTILE(tp, tilePlaneNum);
+		t = TiGetTypeExact(tp);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILE(tp, tilePlaneNum);
+		}
 	    }
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
@@ -2587,15 +2591,19 @@ rightside:
             if (IsSplit(tp))
 	    {
                 t = SplitLeftType(tp);
-		tp->ti_body = (ClientData)(pointertype)((TileType)(spointertype)tp->ti_body & ~TT_SIDE);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILELEFT(tp, tilePlaneNum);
+		}
 	    }
             else
 #endif
-	    t = TiGetTypeExact(tp);
-
-	    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
 	    {
-		PUSHTILE(tp, tilePlaneNum);
+		t = TiGetTypeExact(tp);
+		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		{
+		    PUSHTILE(tp, tilePlaneNum);
+		}
 	    }
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
@@ -2623,20 +2631,51 @@ donesides:
 		    GOTOPOINT(tp, &tile->ti_ll);
 		    plane->pl_hint = tp;
 
+		    if (tp->ti_client != extUnInit) continue;
+
 #ifdef NONMANHATTAN
 		    /* tp and tile should have the same geometry for a contact */
 		    if (IsSplit(tile) && IsSplit(tp))
 		    {           
-			t = (SplitSide(tile)) ? SplitRightType(tp):
-				SplitLeftType(tp);
+			if (SplitSide(tile))
+			{
+			    t = SplitRightType(tp);
+			    if (TTMaskHasType(mask, t))
+			    {
+				PUSHTILERIGHT(tp, pNum);
+			    }
+			}
+			else
+			{
+			    t = SplitLeftType(tp);
+			    if (TTMaskHasType(mask, t))
+			    {
+				PUSHTILELEFT(tp, pNum);
+			    }
+			}
 		    }           
-		    else        
-#endif              
-		    t = TiGetTypeExact(tp);
-
-		    if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
+		    else if (IsSplit(tp))
 		    {
-			PUSHTILE(tp, pNum);
+			/* Need to test both sides of the tile */
+			t = SplitRightType(tp);
+			if (TTMaskHasType(mask, t))
+			{
+			    PUSHTILERIGHT(tp, pNum);
+			}
+			t = SplitLeftType(tp);
+			if (TTMaskHasType(mask, t))
+			{
+			    PUSHTILELEFT(tp, pNum);
+			}
+		    }
+		    else
+#endif              
+		    {
+			t = TiGetTypeExact(tp);
+			if (TTMaskHasType(mask, t))
+			{
+			    PUSHTILE(tp, pNum);
+			}
 		    }
 		}
 	}
