@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtPerim.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtPerim.c,v 1.3 2007/10/15 22:07:21 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -35,12 +35,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "extract/extract.h"
 #include "extract/extractInt.h"
 
-#define	extInMask(bp, tbl) \
-    TTMaskHasType(&tbl[TiGetType((bp)->b_inside)], TiGetType((bp)->b_outside))
-
-
 #define	POINTEQUAL(p, q)	((p)->p_x == (q)->p_x && (p)->p_y == (q)->p_y)
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -76,17 +72,19 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
  */
 
 int
-extEnumTilePerim(tpIn, mask, func, cdata)
+extEnumTilePerim(tpIn, mask, pNum, func, cdata)
     Tile *tpIn;
     TileTypeBitMask mask;	/* Note: this is not a pointer */
+    int pNum;			/* Plane of perimeter */
     int (*func)();
     ClientData cdata;
 {
     Tile *tpOut;
-    int perimLength;
+    u_int perimLength;
     Boundary b;
 
     b.b_inside = tpIn;
+    b.b_plane = pNum;
     perimLength = 0;
 
 	/* Top */
