@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtSubtree.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtSubtree.c,v 1.2 2007/12/24 22:56:44 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -533,11 +533,11 @@ extSubtreeOutputCoupling(ha)
 	ck = (CoupleKey *) he->h_key.h_words;
 
 	NODETOTILE(ck->ck_1, &ha->ha_cumFlat, tp);
-	name = extSubtreeTileToNode(tp, &ha->ha_cumFlat, ha, TRUE);
+	name = extSubtreeTileToNode(tp, ck->ck_1->nreg_pnum, &ha->ha_cumFlat, ha, TRUE);
 	fprintf(ha->ha_outf, "cap \"%s\" ", name);
 
 	NODETOTILE(ck->ck_2, &ha->ha_cumFlat, tp);
-	name = extSubtreeTileToNode(tp, &ha->ha_cumFlat, ha, TRUE);
+	name = extSubtreeTileToNode(tp, ck->ck_2->nreg_pnum, &ha->ha_cumFlat, ha, TRUE);
 	fprintf(ha->ha_outf, "\"%s\" %lg\n", name, cap);
     }
 }
@@ -703,8 +703,9 @@ extSubtreeFunc(scx, ha)
  */
 
 char *
-extSubtreeTileToNode(tp, et, ha, doHard)
+extSubtreeTileToNode(tp, pNum, et, ha, doHard)
     Tile *tp;	/* Tile whose node name is to be found */
+    int pNum;	/* Plane of the tile */
     ExtTree *et;	/* Yank buffer to search */
     HierExtractArg *ha;	/* Extraction context */
     bool doHard;	/* If TRUE, we look up this node's name the hard way
@@ -718,7 +719,6 @@ extSubtreeTileToNode(tp, et, ha, doHard)
 	"Cannot find the name of this node (probable extractor error)";
     CellDef *parentDef = ha->ha_parentUse->cu_def;
     LabRegion *reg;
-    int pNum;
     Rect r;
     TileType ttype;
 
@@ -753,7 +753,6 @@ extSubtreeTileToNode(tp, et, ha, doHard)
 #endif
 	ttype = TiGetTypeExact(tp);
 
-	pNum = DBPlane(ttype);
 	if (pNum >= PL_PAINTBASE)
 	{
 #ifdef NONMANHATTAN
@@ -789,7 +788,7 @@ extSubtreeTileToNode(tp, et, ha, doHard)
     /* We have to do it the hard way */
     if (!doHard) return ((char *) NULL);
     if (extHasRegion(tp, extUnInit)
-	    && (reg = extSubtreeHardNode(tp, et, ha)))
+	    && (reg = extSubtreeHardNode(tp, pNum, et, ha)))
     {
 	if (ExtDoWarn & EXTWARN_LABELS)
 	{
@@ -867,8 +866,9 @@ extConnFindFunc(tp, preg)
  */
 
 LabRegion *
-extSubtreeHardNode(tp, et, ha)
+extSubtreeHardNode(tp, pNum, et, ha)
     Tile *tp;
+    int pNum;
     ExtTree *et;
     HierExtractArg *ha;
 {
@@ -895,7 +895,7 @@ extSubtreeHardNode(tp, et, ha)
 
     arg.hw_ha = ha;
     arg.hw_label = (Label *) NULL;
-    arg.hw_mask = DBPlaneTypes[DBPlane(ttype)];
+    arg.hw_mask = DBPlaneTypes[pNum];
     TTMaskAndMask(&arg.hw_mask, &DBConnectTbl[ttype]);
     arg.hw_tpath.tp_last = &labelBuf[sizeof labelBuf - 3];
     arg.hw_tpath.tp_first = arg.hw_tpath.tp_next = labelBuf;
