@@ -4,7 +4,7 @@
  * Defines things shared internally by the extract module of Magic,
  * but not generally needed outside the extract module.
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/extract/extractInt.h,v 1.7 2007/10/15 22:07:21 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-7.5/extract/extractInt.h,v 1.10 2008/01/17 21:16:36 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -340,6 +340,10 @@ typedef struct
     Rect	 ha_subArea;	/* Area of ha_subUse inside the interaction
 				 * area, i.e, contained within ha_interArea.
 				 */
+    Tile	*hierOneTile;	/* Used in ExtHier.c, tile from extHierOneFlat */
+    int		hierPNum;	/* Used in ExtHier.c, plane of tile above */
+    TileType	hierType;	/* Used in ExtHier.c, type of tile above */
+    int		hierPNumBelow;	/* Used in ExtHier.c, plane of tile below */
 } HierExtractArg;
 
 /*
@@ -874,8 +878,8 @@ extern ClientData extUnInit;
 
 #define	PUSHTILE(tp, pl) \
 	(tp)->ti_client = VISITPENDING; \
-	STACKPUSH((ClientData)(pointertype)(pl | ((TileType)(spointertype)(tp)->ti_body & TT_SIDE)), \
-		extNodeStack); \
+	STACKPUSH((ClientData)(pointertype)(pl | \
+		((TileType)(spointertype)(tp)->ti_body & TT_SIDE)), extNodeStack); \
 	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
 
 #define POPTILE(tp, pl) \
@@ -888,6 +892,30 @@ extern ClientData extUnInit;
 	else \
 	   TiSetBody((tp), TiGetTypeExact(tp) & (~TT_SIDE))
 
+/* Variations of "pushtile" to force a specific value on TT_SIDE */
+
+#define PUSHTILEBOTTOM(tp, pl) \
+	(tp)->ti_client = VISITPENDING; \
+	STACKPUSH((ClientData)(pointertype)(pl | \
+		((SplitDirection(tp)) ? 0 : TT_SIDE)), extNodeStack) ;\
+	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
+
+#define PUSHTILETOP(tp, pl) \
+	(tp)->ti_client = VISITPENDING; \
+	STACKPUSH((ClientData)(pointertype)(pl | \
+		((SplitDirection(tp)) ? TT_SIDE : 0)), extNodeStack) ;\
+	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
+
+#define PUSHTILELEFT(tp, pl) \
+	(tp)->ti_client = VISITPENDING; \
+	STACKPUSH((ClientData)(pointertype)(pl), extNodeStack); \
+	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
+
+#define PUSHTILERIGHT(tp, pl) \
+	(tp)->ti_client = VISITPENDING; \
+	STACKPUSH((ClientData)(pointertype)(pl | TT_SIDE), extNodeStack); \
+	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
+
 #else
 
 #define	PUSHTILE(tp, pl) \
