@@ -12,6 +12,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <signal.h>
 #include <sys/time.h>
 #include <X11/Xlib.h>
@@ -39,6 +40,7 @@
 void sigSetAction(int, sigRetVal (*)(int));
 sigRetVal TimeOut(int);
 void SetTimeOut();
+void ParseEvent(XEvent *);
 
 /*
  * Declaration of global variables and procedure prototypes
@@ -138,8 +140,11 @@ ParseEvent (event)
 
 	if (IsModifierKey(keysym)) return;  /* Don't send modifiers as events */
 
-	keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
+	keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
+	if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask | Mod3Mask
+				| Mod4Mask | Mod5Mask))
+	    keymod |= Mod1Mask;
 
 	if (nbytes == 0)  	/* No ASCII equivalent to string */
 	{
@@ -288,10 +293,9 @@ MapWindow(int signo)
     if (read(readPipe, (char *)&window, sizeof(Window)) == sizeof(Window))
     {
 	XSelectInput(grXdpy, window,
-		     KeyPressMask|ButtonPressMask|
-		     ButtonReleaseMask|ExposureMask|
-		     StructureNotifyMask|
-		     OwnerGrabButtonMask);
+		     KeyPressMask|ButtonPressMask|ButtonReleaseMask|
+		     ExposureMask|StructureNotifyMask|
+		     VisibilityChangeMask|OwnerGrabButtonMask);
 	XSync(grXdpy,1);
     }
     else
