@@ -8,12 +8,8 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#ifdef	SYSV
 #include <string.h>
-#else
-#include <strings.h>
-#endif
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -518,9 +514,16 @@ TOGLEventProc(clientData, xevent)
 
 keys_and_buttons:
 
-		keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
+		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
 
+		/* Change by Philippe---Allows magic to deal with Mac OS-X
+		 * mapping Meta to Mod2 instead of Mod1
+		 */
+		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask | Mod3Mask
+				| Mod4Mask | Mod5Mask))
+		    keymod |= Mod1Mask;
+
 		if (nbytes == 0)	/* No ASCII equivalent */
 		{
 		    keywstate = (keymod << 16) | (keysym & 0xffff);
@@ -590,9 +593,10 @@ keys_and_buttons:
 			        TxSetPoint(KeyPressedEvent->x,
 					grXtransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
-				TxInputRedirect = TX_INPUT_NORMAL;
+				TxInputRedirect = TX_INPUT_PROCESSING;
 				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
 						19, 0);
+				TxInputRedirect = TX_INPUT_NORMAL;
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
