@@ -804,8 +804,11 @@ keys_and_buttons:
 		grCurrent.windowid = Tk_WindowId(wind);
 		grCurrent.mw = mw;
 
-		keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
+		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
+		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask |
+				Mod3Mask | Mod4Mask | Mod5Mask))
+		    keymod |= Mod1Mask;
 
 		if (nbytes == 0)
 		{
@@ -883,9 +886,10 @@ keys_and_buttons:
 			        TxSetPoint(KeyPressedEvent->x,
 					grXToMagic(KeyPressedEvent->y),
 					grCurrent.mw->w_wid);
-				TxInputRedirect = TX_INPUT_NORMAL;
+				TxInputRedirect = TX_INPUT_PROCESSING;
 				Tcl_EvalEx(consoleinterp, "::tkcon::Eval .text",
 						19, 0);
+				TxInputRedirect = TX_INPUT_NORMAL;
 				TxSetPrompt('%');
 
 				Tcl_SaveResult(magicinterp, &state);
