@@ -76,8 +76,11 @@ ParseEvent (event, parentID)
 
 	if (IsModifierKey(keysym)) return;  /* Don't send modifiers as events */
 
-	keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
+	keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
+	if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask | Mod3Mask
+				| Mod4Mask | Mod5Mask))
+	    keymod |= Mod1Mask;
 
 	if (nbytes == 0)  	/* No ASCII equivalent to string */
 	{
@@ -224,7 +227,8 @@ xloop_create(window)
 void
 xloop_end()
 {
-    pthread_cancel(xloop_thread);
+    if (xloop_thread != (pthread_t)0)
+	pthread_cancel(xloop_thread);
 
     /* For whatever reason, calling this used to work, now just	*/
     /* causes magic to hang. . . 				*/
