@@ -16,7 +16,7 @@
  *     * of the United States of America may require an export license.    * 
  *     *********************************************************************
  *
- * rcsid $Header: /usr/cvsroot/magic-7.5/plot/plotInt.h,v 1.4 2008/01/15 23:51:35 tim Exp $
+ * rcsid $Header: /usr/cvsroot/magic-7.5/plot/plotInt.h,v 1.5 2008/04/24 18:57:47 tim Exp $
  */
 
 #ifndef _PLOTINT_H
@@ -156,6 +156,7 @@ extern bool PlotVersTechLine();
 
 extern void PlotColorVersTechInit();
 extern bool PlotColorVersTechLine();
+extern void PlotColorVersTechFinal();
 
 extern void PlotPNMTechInit();
 extern bool PlotPNMTechLine();
@@ -204,6 +205,7 @@ extern int PlotPSMargin;
 
 /* for Versatec (plotVers.c): */
 #ifdef VERSATEC
+extern float PlotVersAlpha;
 extern int PlotVersWidth;
 extern int PlotVersDotsPerInch;
 extern int PlotVersSwathHeight;
