@@ -18,10 +18,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotVers.c,v 1.13 2008/01/29 02:41:57 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotVers.c,v 1.14 2008/04/24 18:57:47 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>		/* For random() */
 #include <string.h>
 
 #include "utils/magic.h"
@@ -41,6 +42,10 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "dbwind/dbwind.h"
 #include "cif/cif.h"		/* for CIFGetOutputScale() */
 
+#include "graphics/graphics.h"	/* for automatic color-versatec color setup */
+#include "utils/styles.h"
+#include "dbwind/dbwtech.h"
+
 #ifdef VERSATEC
 
 /* Library imports: */
@@ -55,7 +60,8 @@ extern int rasFileByteCount;
 typedef struct versatecstyle
 {
     TileTypeBitMask vs_layers;		/* Layers to plot in this style. */
-    Stipple vs_stipple;			/* Stipple pattern to use. */
+    Stipple vs_stipple;			/* Area stipple pattern to use. */
+    Stipple vs_lineStipple;		/* Line stipple pattern to use. */
     int vs_flags;			/* Flag bits, see below. */
     struct versatecstyle *vs_next;	/* Pointer to next style in list. */
 
@@ -100,6 +106,11 @@ char *plotVersatecColorNames[] = {
 
 unsigned char PlotVersPlotType = HPRTL;
 
+float PlotVersAlpha = 1.6;	/* When auto-generating stipples, use
+				 * this value for alpha (color value
+				 * scaling; values > 1 make colors lighter)
+				 */
+
 int PlotVersWidth = 2400;	/* Number of dots across Versatec page.
 				 * Should be a multiple of 32.
 				 */
@@ -237,6 +248,8 @@ PlotVersTechInit()
 
 #endif /* VERSATEC */
 
+#ifdef VERSATEC
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -253,8 +266,6 @@ PlotVersTechInit()
  * ----------------------------------------------------------------------------
  */
 
-#ifdef VERSATEC
-
 void
 PlotColorVersTechInit()
 {
@@ -280,12 +291,138 @@ PlotColorVersTechInit()
 	StrDup(&PlotVersLabelFont, defaultLabelFont);
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *	PlotColorVersTechFinal --
+ *
+ * 	Called once at end of technology file read-in to create a default
+ *	style if none was defined, and to fill in line stipple patterns.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Clears out the list of things to plot.
+ * ----------------------------------------------------------------------------
+ */
+
+void
+PlotColorVersTechFinal()
+{
+    VersatecStyle *new, *style;
+    int cs, i, j, k, l, fill, stipple;
+    int cc, red, green, blue, colors[3];
+
+    if (plotColorVersStyles == NULL)
+    {
+	for (j = 0; j < DBWNumStyles; j++)
+	{
+	    cc = GrStyleTable[j + TECHBEGINSTYLES].color;
+	    GrGetColor(cc, &red, &green, &blue);
+	    colors[0] = 255 - red;
+	    colors[1] = 255 - green;
+	    colors[2] = 255 - blue;
+
+	    for (cc = CYAN; cc <= YELLOW; cc++)
+	    {
+		new = (VersatecStyle *)mallocMagic(sizeof(VersatecStyle));
+		TTMaskZero(&new->vs_layers);
+		TTMaskSetMask(&new->vs_layers, DBWStyleToTypes(j));
+
+		/* Set the Versatec style to border or cross if the	*/
+		/* display style is so defined.				*/
+
+		new->vs_color = cc;
+		new->vs_flags = 0;
+		switch (GrStyleTable[j + TECHBEGINSTYLES].fill)
+		{
+		    case GR_STCROSS:
+			new->vs_flags = VS_CROSS;
+			break;
+		    case GR_STOUTLINE:
+			new->vs_flags = VS_BORDER;
+			break;
+		    case GR_STSTIPPLE:
+			stipple = GrStyleTable[j + TECHBEGINSTYLES].stipple;
+			break;
+		    case GR_STSOLID:
+			stipple = -1;
+			break;
+		}
+
+		/* Randomly fill bits of vs_stipple to match color.	*/
+		/* Mask to the display style stipple mask!		*/
+		/* Display style stipple is 8x8; PlotVers stipple is	*/
+		/* 16 x 16, so we correct for the factor of 2.		*/
+
+		fill = (int)((2.0 / PlotVersAlpha) * (float)colors[cc - CYAN]);
+		for (i = 0; i < 16; i++)
+		{
+		    if (stipple >= 0)
+			l = GrStippleTable[stipple][i >> 1];
+
+		    cs = 0;
+		    for (k = 0; k < 8 * sizeof(int); k++)
+		    {
+			cs <<= 1;
+			if (stipple >= 0)
+			{
+			    if (((l >> ((k % 16) >> 1)) & 0x1) == 0)
+				continue;
+			}
+			if ((random() % 512) < fill) cs |= 0x1;
+		    }
+		    new->vs_stipple[i] = cs;
+		}
+
+		new->vs_next = plotColorVersStyles;
+		plotColorVersStyles = new;
+	    }
+	}
+    }
+
+    for (style = plotColorVersStyles; style != NULL; style = style->vs_next)
+    {
+	fill = 0;
+	for (i = 0; i < 16; i++)
+	{
+	    cs = style->vs_stipple[i];
+	    for (j = 0; j < 8 * sizeof(int); j++)
+	    {
+		if (cs & 0x1) fill++;
+		cs >>= 1;
+	    }
+	}
+
+	/* "fill" is the number of bits of this color out of 512. */
+	/* Now make it 30% darker */
+
+	fill = (512 + 2 * fill) / 3;
+
+	/* Randomly fill bits of vs_lineStipple to the density of "fill" */
+	for (i = 0; i < 16; i++)
+	{
+	    cs = 0;
+	    for (j = 0; j < 8 * sizeof(int); j++)
+	    {
+		cs <<= 1;
+		if ((random() % 512) < fill) cs |= 0x1;
+	    }
+	    style->vs_lineStipple[i] = cs;
+	}
+    }
+}
+
 #else
 
 void
 PlotColorVersTechInit()
 {}
 
+void
+PlotColorVersTechFinal()
+{}
+
 #endif /* VERSATEC */
 
 
@@ -534,7 +671,7 @@ plotTransToSwath(src, dst)
  */
 
 void
-plotVersLine(area, widen, raster)
+plotVersLine(area, widen, raster, stipple)
     Rect *area;			/* The "corner" points of this rectangle
 				 * give the endpoints of the line, in
 				 * Magic root coordinates.
@@ -544,6 +681,7 @@ plotVersLine(area, widen, raster)
 				 * pixels wide, etc.
 				 */
     Raster *raster;		/* Raster to write to */
+    Stipple *stipple;		/* Pattern to use to get the right line color */
 {
     Rect swathArea;
 
@@ -558,10 +696,10 @@ plotVersLine(area, widen, raster)
 	GEOCLIP(&swathArea, &swathClip);
 	if ((swathArea.r_xbot <= swathArea.r_xtop) &&
 		(swathArea.r_ybot <= swathArea.r_ytop))
-	    PlotFillRaster(raster, &swathArea, PlotBlackStipple);
+	    PlotFillRaster(raster, &swathArea, stipple);
     }
     else
-    PlotRastFatLine(raster, &swathArea.r_ll, &swathArea.r_ur, widen);
+    PlotRastFatLine(raster, &swathArea.r_ll, &swathArea.r_ur, widen, stipple);
 }
 
 /*
@@ -583,7 +721,7 @@ plotVersLine(area, widen, raster)
  */
 
 void
-plotVersRect(area, widen, raster)
+plotVersRect(area, widen, raster, stipple)
     Rect *area;	/* Rectangular area to draw, in root
 				 * coordinates.
 				 */
@@ -593,6 +731,7 @@ plotVersRect(area, widen, raster)
 				 * side.
 				 */
     Raster *raster;		/* Raster to plot to */
+    Stipple *stipple;		/* Pattern to use to get the right line color */
 {
     Rect side;
 
@@ -602,7 +741,7 @@ plotVersRect(area, widen, raster)
     {
 	side = *area;
 	side.r_ytop = side.r_ybot;
-	plotVersLine(&side, widen, raster);
+	plotVersLine(&side, widen, raster, stipple);
 
 	/* Now the top side, if it doesn't coincide with the bottom. */
 
@@ -610,7 +749,7 @@ plotVersRect(area, widen, raster)
 	{
 	    side = *area;
 	    side.r_ybot = side.r_ytop;
-	    plotVersLine(&side, widen, raster);
+	    plotVersLine(&side, widen, raster, stipple);
 	}
     }
 
@@ -620,7 +759,7 @@ plotVersRect(area, widen, raster)
     {
 	side = *area;
 	side.r_xtop = side.r_xbot;
-	plotVersLine(&side, widen, raster);
+	plotVersLine(&side, widen, raster, stipple);
 
 	/* Now the right side, if it doesn't coincide with the left. */
 
@@ -628,7 +767,7 @@ plotVersRect(area, widen, raster)
 	{
 	    side = *area;
 	    side.r_xbot = side.r_xtop;
-	    plotVersLine(&side, widen, raster);
+	    plotVersLine(&side, widen, raster, stipple);
 	}
     }
 }
@@ -678,7 +817,7 @@ plotVersTile(tile, cxp)
 			&& ((swathArea.r_ytop - swathArea.r_ybot) > 6))
 	    {
 		Rect r2;
-		plotVersLine(&rootArea, 0, raster);
+		plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
 		r2.r_xtop = rootArea.r_xbot;
 		r2.r_ybot = rootArea.r_ybot;
 		r2.r_xbot = rootArea.r_xtop;
@@ -708,7 +847,7 @@ plotVersTile(tile, cxp)
 	    r.r_xbot = r.r_xtop;
 	    r.r_xtop = rootArea.r_xbot;
 	}
-	plotVersLine(&r, 0, raster);
+	plotVersLine(&r, 0, raster, curStyle->vs_lineStipple);
     }
     else
     {
@@ -751,7 +890,7 @@ plotVersTile(tile, cxp)
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster);
+	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
 	}
     }
 
@@ -778,7 +917,7 @@ searchleft:
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
 	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster);
+	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
 	}
     }
 
@@ -803,7 +942,7 @@ searchtop:
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster);
+	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
 	}
     }
 
@@ -828,7 +967,7 @@ searchright:
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
 	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster);
+	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
 	}
     }
     return 0;
@@ -950,7 +1089,7 @@ plotVersLabel(scx, label, tpath, raster)
     {
 	/* Line or rectangle.  Draw outline. */
 
-	plotVersRect(&rootArea, 1, raster);
+	plotVersRect(&rootArea, 1, raster, PlotBlackStipple);
     }
 
     /* Output the text for the label.  Before outputting the label,
