@@ -4,7 +4,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResConDCS.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResConDCS.c,v 1.2 2008/02/10 19:06:32 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -234,19 +234,25 @@ dbcConnectFuncDCS(tile, cx)
 
     if (DBIsContact(loctype))
     {
-	TileTypeBitMask *rMask = DBResidueMask(loctype);
+	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
 
 	TTMaskSetOnlyType(&notConnectMask, loctype);
 
 	/* Differenct contact types may share residues (6/18/04) */     
 	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	    if (TTMaskIntersect(rMask, DBResidueMask(ctype)))
+	{
+	    cMask = DBResidueMask(ctype);
+	    if (TTMaskIntersect(rMask, cMask))
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	/* The mask of contact types must include all stacked contacts */
 	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)   
-	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+	{
+	    cMask = DBResidueMask(ctype);
+	    if TTMaskHasType(cMask, loctype)
 		TTMaskSetType(&notConnectMask, ctype);
+	}
 
 	TTMaskCom(&notConnectMask);
     }
