@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMakeRes.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMakeRes.c,v 1.2 2006/11/10 18:17:53 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -793,12 +793,14 @@ ResDoContacts(contact, nodes, resList)
     resNode	 *resptr;
     cElement	 *ccell;
     int		 tilenum, squaresx, squaresy, viawidth;
+    int 	 minside, spacing, border;
     resResistor	 *resistor;
     resElement	 *element;
     static int	 too_small = 1;
      
-    if (ExtCurStyle->exts_viaResist[contact->cp_type] == 0 ||
-		ExtCurStyle->exts_viaSize[contact->cp_type] == 0)
+    minside = CIFGetContactSize(contact->cp_type, &viawidth, &spacing, &border);
+
+    if ((ExtCurStyle->exts_viaResist[contact->cp_type] == 0) || (viawidth == 0))
     {
 	int x = contact->cp_center.p_x;
 	int y = contact->cp_center.p_y;
@@ -825,9 +827,6 @@ ResDoContacts(contact, nodes, resList)
     }
     else
     {
-	int minside = ExtCurStyle->exts_viaSize[contact->cp_type] +
-		2 * ExtCurStyle->exts_viaBorder[contact->cp_type];
-
 	if ((contact->cp_width < minside) || (contact->cp_height < minside))
 	{
 	    if (too_small)
@@ -838,15 +837,19 @@ ResDoContacts(contact, nodes, resList)
 	        too_small = 0;
 	    }
 	    squaresx = squaresy = 1;
-	    viawidth = ExtCurStyle->exts_viaSize[contact->cp_type];
 	}
 	else
 	{
-	    viawidth = ExtCurStyle->exts_viaSize[contact->cp_type] +
-			ExtCurStyle->exts_viaSpacing[contact->cp_type];
-			
-	    squaresx = 1 + ((contact->cp_width - minside) / viawidth);
-	    squaresy = 1 + ((contact->cp_height - minside) / viawidth);
+	    viawidth += spacing;
+	    squaresx = (contact->cp_width - minside) / viawidth;
+	    squaresx *= ExtCurStyle->exts_unitsPerLambda;
+	    squaresx /= viawidth;
+	    squaresx++;
+
+	    squaresy = (contact->cp_height - minside) / viawidth;
+	    squaresy *= ExtCurStyle->exts_unitsPerLambda;
+	    squaresy /= viawidth;
+	    squaresy++;
 	}
         for (tilenum=0; tilenum < contact->cp_currentcontact; tilenum++)
 	{
