@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.11 2008/02/02 01:18:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.16 2008/05/19 17:18:39 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -112,6 +112,7 @@ SelectInit()
 	ASSERT(SelectDef != (CellDef *) NULL, "SelectInit");
 	DBCellSetAvail(SelectDef);
 	SelectDef->cd_flags |= CDINTERNAL;
+	TTMaskZero(&SelectDef->cd_types);
     }
     SelectUse = DBCellNewUse(SelectDef, (char *) NULL);
     DBSetTrans(SelectUse, &GeoIdentityTransform);
@@ -171,18 +172,19 @@ SelectClear()
 
     scx.scx_area = SelectDef->cd_bbox;
 
-    /* Special handling when the cell is named __SELECT_NET__ 	*/
+    /* Special handling when the cell is flagged as a net selection */
 
-    if (!strncmp(SelectDef->cd_name + 9, "NET", 3))
+    if (SelectUse->cu_flags & CU_SELECT_NET)
     {
 	SelNetRememberForUndo((CellDef *)NULL, (Point *)NULL,
 			TT_SPACE, FALSE, FALSE);
-	DBCellRenameDef(SelectDef, "__SELECT__");
+	SelectUse->cu_flags = 0;
 	DBCellClearDef(SelectDef);
     }
     else
     {
 	SelRememberForUndo(TRUE, (CellDef *) NULL, (Rect *) NULL);
+	SelectUse->cu_flags = 0;
 
 	/* Erase all the paint from the select cell. */
 	DBEraseMask(SelectDef, &TiPlaneRect, &DBAllButSpaceBits);
@@ -221,6 +223,7 @@ SelectClear()
 
 	SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
     }
+    TTMaskZero(&SelectDef->cd_types);
 
     /* Reset the transform, if we have been moving the selection around */
     GeoTransRect(&SelectUse->cu_transform, &scx.scx_area, &r);
@@ -309,6 +312,8 @@ SelectArea(scx, types, xMask)
 
     (void) DBCellCopyAllPaint(scx, types, xMask, SelectUse);
 
+    SelectDef->cd_types = *types;	/* Remember what types were requested */
+
     /* Select labels. */
 
     if (TTMaskHasType(types, L_LABEL))
@@ -357,12 +362,24 @@ SelectArea(scx, types, xMask)
  * Side effects:
  *	None.
  *
+ * Notes:
+ *	This routine is O(2^N) in the worst-case (caused by stair-stepped
+ *	geometry at single-unit spacing).  Limiting the recursion level to
+ *	22 causes the routine to fail to find the maximum-sized rectangle
+ *	in such cases but also ensures a completion time of less than a
+ *	second or so on a decent machine (e.g., 2GHz CPU).  The inability
+ *	to find the maximum-area rectangle is not really such a big deal
+ *	in these (rare) cases.  However, there must be better area-
+ *	partitioning algorithms out there, and a replacement would be nice.
+ *
  * ----------------------------------------------------------------------------
  */
 
+#define MAX_RECURSION_LEVEL 22
+
 void
 selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
-	bestMax, bestChunk)
+	bestMax, bestChunk, level)
     Plane *plane;			/* Plane on which to hunt for chunk. */
     TileTypeBitMask *wrongTypes;	/* Types that are not allowed to be
 					 * part of the chunk.
@@ -388,12 +405,18 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 					 * so far, if we find one better than
 					 * bestMin and bestMax.
 					 */
+    int level;				/* Recursion level, to avoid deep
+					 * recursion (this routine is worst-
+					 * case O(2^N)!)
+					 */
 {
     Rect smaller;
     int min, max;
     extern int selChunkFunc();
     Rect wrong;
 
+    if (level == MAX_RECURSION_LEVEL) return;
+
     /* If the search area is already smaller than the chunk to beat,
      * there's no point in even examining this chunk.
      */
@@ -445,7 +468,7 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	smaller.r_xbot = wrong.r_xtop;
     else goto tryY;  /* Bad material overlaps containedArea in x. */
     selFindChunk(plane, wrongTypes, &smaller, containedArea,
-	    bestMin, bestMax, bestChunk);
+	    bestMin, bestMax, bestChunk, level + 1);
     
 
     /* Also try reducing the y-range to see if that works better. */
@@ -457,7 +480,7 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	smaller.r_ybot = wrong.r_ytop;
     else return;  /* Bad material overlaps containedArea in y. */
     selFindChunk(plane, wrongTypes, &smaller, containedArea,
-	    bestMin, bestMax, bestChunk);
+	    bestMin, bestMax, bestChunk, level + 1);
 }
 
 #ifdef NONMANHATTAN
@@ -596,6 +619,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 #endif
 
+    bestMin = bestMax = 0;
+    bestChunk = GeoNullRect;
+
     GEO_EXPAND(&newscx.scx_area, INITIALSIZE, &newscx.scx_area);
     while (TRUE)
     {
@@ -610,12 +636,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 	/* Now find the best chunk in the area. */
 
-	bestMin = bestMax = 0;
-	bestChunk = GeoNullRect;
-
 	selFindChunk(Select2Def->cd_planes[DBPlane(type)],
 	    &wrongTypes, &newscx.scx_area, &scx->scx_area,
-	    &bestMin, &bestMax, &bestChunk);
+	    &bestMin, &bestMax, &bestChunk, 0);
 	if (GEO_RECTNULL(&bestChunk))
 	{
 	    /* No chunk was found, so return. */
@@ -656,6 +679,10 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 chunkdone:
 
+    /* Flag the cell as a chunk selection so we can treat       */
+    /* the contents differently when doing a move op.           */
+
+    SelectUse->cu_flags |= CU_SELECT_CHUNK; 
     UndoEnable();
     if (less)
       {
@@ -770,10 +797,7 @@ SelectRegion(scx, type, xMask, pArea, less)
 
     SelRememberForUndo(FALSE, SelectRootDef, &Select2Def->cd_bbox);
 
-    /* SelectDef should not need to recompute the bounding box;	*/
-    /* only to copy it (transformed) from Select2Def.		*/
-    /* DBReComputeBbox(SelectDef); */
-    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
     DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
@@ -872,20 +896,15 @@ SelectNet(scx, type, xMask, pArea, less)
 			(Rect *) NULL);
     }
 
-    /* Rename the cell to "__SELECT_NET__" so that we can treat */
-    /* it differently when it comes time to unselect it.	*/
+    /* Set the cell use flags to mark this as a net selection,  */
+    /* so we can treat it differently when it comes time to     */
+    /* unselect it.						*/
 
-    DBCellRenameDef(SelectDef, "__SELECT_NET__");
+    SelectUse->cu_flags |= CU_SELECT_NET;
     UndoEnable(); 
 
     /* Display the newly-selected material. */
-    /* SelectDef should not need to recompute the bounding box;	*/
-    /* only to copy it (transformed) from Select2Def, and then	*/
-    /* transfer this bounding box to the use.			*/
-    /* ---Nishit, 7/6/04					*/
-    /* DBReComputeBbox(SelectDef); */
-
-    GeoTransRect(&scx->scx_trans, &Select2Def->cd_bbox, &SelectDef->cd_bbox);
+    DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
     DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
@@ -1215,7 +1234,7 @@ selACPaintFunc2(tile, selACarg)
     Rect *editClip = selACarg->editClip;
     Rect area, selArea;
     TileType type = TiGetTypeExact(tile);
-    TileTypeBitMask tmask;
+    TileTypeBitMask tmask, *rmask;
     TileType ttype, rtype;
 
 #ifdef NONMANHATTAN
@@ -1245,8 +1264,13 @@ selACPaintFunc2(tile, selACarg)
 	else
 	    rtype = type;
 
-	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
-	    rtype = ttype;
+	if (rtype >= DBNumUserLayers)
+	{
+	    rmask = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask, ttype))
+		rtype = ttype;
+	}
+
 	TTMaskSetOnlyType(&tmask, rtype);
 
 	type = (dinfo & TT_SIDE) ? (rtype << 14) : rtype;
@@ -1339,15 +1363,23 @@ topbottom:
 	else
 	    rtype = type;
 
-	if (rtype >= DBNumUserLayers && TTMaskHasType(DBResidueMask(rtype), ttype))
-	    rtype = ttype;
+	if (rtype >= DBNumUserLayers)
+	{
+	    rmask = DBResidueMask(rtype);
+	    if (TTMaskHasType(rmask, ttype))
+		rtype = ttype;
+	}
 	TTMaskSetOnlyType(&tmask, rtype);
     }
 #else	/* !NONMANHATTAN */
 
     ttype = selACarg->ttype;
-    if (type >= DBNumUserLayers && TTMaskHasType(DBResidueMask(type), ttype))
+    if (type >= DBNumUserLayers)
+    {
+	rmask = DBResidueMask(type);
+	if (TTMaskHasType(rmask, ttype))
 	    type = ttype;
+    }
     TTMaskSetOnlyType(&tmask, type);
 
 #endif
