@@ -90,17 +90,6 @@ proc magic::promptload {type} {
 		{{Magic {.mag {.mag}}} {"All files" {*}}}]
 	    if {$Layoutfilename != ""} {
 		magic::load [file root $Layoutfilename]
-
-		# Append path to cell search path if it's not there already
-
-		if {[string index $Layoutfilename 0] != "/"} {
-		   set $Layoutfilename "./$Layoutfilename"
-		}
-		set sidx [string last "/" $Layoutfilename]
-		if {$sidx > 0} {
-		   set cellpath [string range $Layoutfilename 0 $sidx]
-		   magic::path cell +$cellpath
-		}
 	    }
 	  }
    }
@@ -532,43 +521,43 @@ proc magic::techmanager {{option "update"}} {
    .techmgr.lambda1.lval1 insert end $tech0
 }
 
-proc magic::caption {winpath {subcommand {}}} {
+proc magic::captions {{subcommand {}}} {
    global Opts
 
    if {$subcommand != {} && $subcommand != "writeable" && $subcommand != "load"} {
       return
    }
-   if {$winpath == ""} {
-      return
-   }
+   set winlist [magic::windownames layout]
+   foreach winpath $winlist {
 
-   set framename [winfo parent $winpath]
-   set caption [$winpath windowcaption]
-   set subcaption1 [lindex $caption 0]
-   set techname [tech name]
-   if {[catch {set Opts(tool)}]} {
-      set Opts(tool) unknown
+      set framename [winfo parent $winpath]
+      set caption [$winpath windowcaption]
+      set subcaption1 [lindex $caption 0]
+      set techname [tech name]
+      if {[catch {set Opts(tool)}]} {
+         set Opts(tool) unknown
+      }
+      if {[lindex $caption 1] == "EDITING"} {
+         set subcaption2 [lindex $caption 2]
+      } else {
+         # set subcaption2 [join [lrange $caption 1 end]]
+         set subcaption2 $caption
+      }
+      ${framename}.titlebar.caption configure -text \
+	   "Loaded: ${subcaption1} Editing: ${subcaption2} Tool: $Opts(tool) \
+	   Technology: ${techname}"
    }
-   if {[lindex $caption 1] == "EDITING"} {
-      set subcaption2 [lindex $caption 2]
-   } else {
-      # set subcaption2 [join [lrange $caption 1 end]]
-      set subcaption2 $caption
-   }
-   ${framename}.titlebar.caption configure -text \
-	"Loaded: ${subcaption1} Editing: ${subcaption2} Tool: $Opts(tool) \
-	Technology: ${techname}"
 }
 
 # Allow captioning in the title window by tagging the "load" and "edit" commands
 # Note that the "box" tag doesn't apply to mouse-button events, so this function
 # is duplicated by Tk binding of mouse events in the layout window.
 
-magic::tag load "magic::cellmanager; magic::caption %W"
+magic::tag load "magic::cellmanager; magic::captions"
 magic::tag getcell "magic::cellmanager"
-magic::tag edit "magic::caption %W"
-magic::tag save "magic::caption %W"
-magic::tag down "magic::caption %W"
+magic::tag edit "magic::captions"
+magic::tag save "magic::captions"
+magic::tag down "magic::captions"
 magic::tag box "magic::boxview %W %1"
 magic::tag move "magic::boxview %W"
 magic::tag scroll "magic::scrollupdate %W"
@@ -576,7 +565,7 @@ magic::tag view "magic::scrollupdate %W"
 magic::tag zoom "magic::scrollupdate %W"
 magic::tag findbox "magic::scrollupdate %W"
 magic::tag see "magic::toolupdate %W %1 %2"
-magic::tag tech "magic::techrebuild %W %1; magic::caption %W %1"
+magic::tag tech "magic::techrebuild %W %1; magic::captions %1"
 magic::tag drc "magic::drcupdate %1"
 magic::tag path "magic::techmanager update"
 if {$MagicBLT} {
@@ -585,7 +574,7 @@ if {$MagicBLT} {
    magic::tag cif "magic::mgrupdate %W %1"
    magic::tag gds "magic::mgrupdate %W %1"
 } else {
-   magic::tag cellname "magic::caption %W %1"
+   magic::tag cellname "magic::captions %1"
 }
 
 # This should be a list. . . do be done later
@@ -794,13 +783,13 @@ if {$MagicBLT} {
    proc magic::mgrupdate {win {cmdstr ""}} {
       if {${cmdstr} == "read"} {
 	 magic::cellmanager
-	 magic::caption $win
+	 magic::captions
 	 magic::techmanager update
       } elseif {${cmdstr} == "delete" || ${cmdstr} == "rename"} {
 	 magic::cellmanager
-	 magic::caption $win
+	 magic::captions
       } elseif {${cmdstr} == "writeable"} {
-	 magic::caption $win
+	 magic::captions
       }
    }
 }
@@ -1213,6 +1202,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		} else { \
 		grid forget ${framename}.toolbar } }]
 
+   .winmenu add radio -label ${framename} -variable Opts(target) -value ${winname}
    if {$MagicBLT} {
       $m add check -label "Cell Manager" -variable Opts(cellmgr) \
 		-command [subst { magic::cellmanager create; \
@@ -1220,13 +1210,11 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		wm deiconify .cellmgr ; raise .cellmgr \
 		} else { \
 		wm withdraw .cellmgr } }]
-
-      .winmenu add radio -label ${framename} \
-		-variable Opts(target) -value ${winname} \
-		-command ".cellmgr.target.list configure \
+      .winmenu entryconfigure last -command ".cellmgr.target.list configure \
 		-text ${framename}"
    }
 
+
    $m add check -label "Tech Manager" -variable Opts(techmgr) \
 	-command [subst { magic::techmanager create; \
 		if { \$Opts(techmgr) } { \
@@ -1278,7 +1266,7 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    update idletasks
    ${winname} magic::view
 
-   magic::caption ${winname}
+   magic::captions
 
    # Remove "open" and "close" macros so they don't generate non-GUI
    # windows or (worse) blow away the window inside the GUI frame
@@ -1309,17 +1297,19 @@ proc magic::closewrapper { framename } {
    global Opts
 
    # Remove this window from the target list in cellmanager
-   if {$MagicBLT} {
-      if {![catch {wm state .cellmgr}]} {
-         if { $Opts(target) == "${framename}.magic" } {
-	    set Opts(target) "default"
+
+   if { $Opts(target) == "${framename}.magic" } {
+      set Opts(target) "default"
+      if {$MagicBLT} {
+         if {![catch {wm state .cellmgr}]} {
 	    .cellmgr.target.list configure -text "default"
-         } 
-         set idx [.cellmgr.target.list.winmenu index $framename ]
-         .cellmgr.target.list.winmenu delete $idx
+	 }
       }
    }
 
+   set idx [.winmenu index $framename ]
+   .winmenu delete $idx
+
    ${framename}.magic magic::closewindow
    destroy $framename
 }
